package com.sun.org.apache.xpath.internal.jaxp;

import com.sun.org.apache.xpath.internal.functions.FuncExtFunction;
import com.sun.org.apache.xml.internal.utils.WrappedRuntimeException;
import javax.xml.xpath.XPathFunctionException;
import com.sun.org.apache.xpath.internal.objects.XObject;
import com.sun.org.apache.xpath.internal.objects.XNodeSet;
import java.util.ArrayList;
import java.util.Vector;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPathFunction;
import com.sun.org.apache.xpath.internal.res.XPATHMessages;
import javax.xml.namespace.QName;
import jdk.xml.internal.JdkXmlFeatures;
import javax.xml.xpath.XPathFunctionResolver;
import com.sun.org.apache.xpath.internal.ExtensionsProvider;

public class JAXPExtensionsProvider implements ExtensionsProvider
{
    private final XPathFunctionResolver resolver;
    private boolean extensionInvocationDisabled;
    
    public JAXPExtensionsProvider(XPathFunctionResolver resolver) {
        this.extensionInvocationDisabled = false;
        this.resolver = resolver;
        this.extensionInvocationDisabled = false;
    }
    
    public JAXPExtensionsProvider(XPathFunctionResolver resolver, boolean featureSecureProcessing, JdkXmlFeatures featureManager) {
        this.extensionInvocationDisabled = false;
        this.resolver = resolver;
        if (featureSecureProcessing && !featureManager.getFeature(JdkXmlFeatures.XmlFeature.ENABLE_EXTENSION_FUNCTION)) {
            this.extensionInvocationDisabled = true;
        }
    }
    
    @Override
    public boolean functionAvailable(String ns, String funcName) throws TransformerException {
        try {
            if (funcName == null) {
                String fmsg = XPATHMessages.createXPATHMessage("ER_ARG_CANNOT_BE_NULL", new Object[] { "Function Name" });
                throw new NullPointerException(fmsg);
            }
            QName myQName = new QName(ns, funcName);
            XPathFunction xpathFunction = this.resolver.resolveFunction(myQName, 0);
            return xpathFunction != null;
        }
        catch (Exception e) {
            return false;
        }
    }
    
    @Override
    public boolean elementAvailable(String ns, String elemName) throws TransformerException {
        return false;
    }
    
    public Object extFunction(String ns, String funcName, Vector argVec, Object methodKey) throws TransformerException {
        try {
            if (funcName == null) {
                String fmsg2 = XPATHMessages.createXPATHMessage("ER_ARG_CANNOT_BE_NULL", new Object[] { "Function Name" });
                throw new NullPointerException(fmsg2);
            }
            QName myQName = new QName(ns, funcName);
            if (this.extensionInvocationDisabled) {
                String fmsg = XPATHMessages.createXPATHMessage("ER_EXTENSION_FUNCTION_CANNOT_BE_INVOKED", new Object[] { myQName.toString() });
                throw new XPathFunctionException(fmsg);
            }
            int arity = argVec.size();
            XPathFunction xpathFunction = this.resolver.resolveFunction(myQName, arity);
            ArrayList argList = new ArrayList(arity);
            for (int i = 0; i < arity; ++i) {
                Object argument = argVec.elementAt(i);
                if ((argument instanceof XNodeSet)) {
                    argList.add(i, ((XNodeSet)argument).nodelist());
                }
                else if ((argument instanceof XObject)) {
                    Object passedArgument = ((XObject)argument).object();
                    argList.add(i, passedArgument);
                }
                else {
                    argList.add(i, argument);
                }
            }
            return xpathFunction.evaluate(argList);
        }
        catch (XPathFunctionException xfe) {
            throw new WrappedRuntimeException(xfe);
        }
        catch (Exception e) {
            throw new TransformerException(e);
        }
    }
    
    public Object extFunction(FuncExtFunction extFunction, Vector argVec) throws TransformerException {
        try {
            String namespace = extFunction.getNamespace();
            String functionName = extFunction.getFunctionName();
            int arity = extFunction.getArgCount();
            QName myQName = new QName(namespace, functionName);
            if (this.extensionInvocationDisabled) {
                String fmsg = XPATHMessages.createXPATHMessage("ER_EXTENSION_FUNCTION_CANNOT_BE_INVOKED", new Object[] { myQName.toString() });
                throw new XPathFunctionException(fmsg);
            }
            XPathFunction xpathFunction = this.resolver.resolveFunction(myQName, arity);
            ArrayList argList = new ArrayList(arity);
            for (int i = 0; i < arity; ++i) {
                Object argument = argVec.elementAt(i);
                if ((argument instanceof XNodeSet)) {
                    argList.add(i, ((XNodeSet)argument).nodelist());
                }
                else if ((argument instanceof XObject)) {
                    Object passedArgument = ((XObject)argument).object();
                    argList.add(i, passedArgument);
                }
                else {
                    argList.add(i, argument);
                }
            }
            return xpathFunction.evaluate(argList);
        }
        catch (XPathFunctionException xfe) {
            throw new WrappedRuntimeException(xfe);
        }
        catch (Exception e) {
            throw new TransformerException(e);
        }
    }
}
